import { isUndefined } from '@collectable/core';
import { NONE, isNone, editLeftChild, editRightChild } from './node';
import { setChild, updateCount } from './ops';
export class PathNode {
    constructor(node, parent, next) {
        this.node = node;
        this.parent = parent;
        this.next = next;
    }
    static next(node, parent, next) {
        var p = PathNode.cache;
        if (p.isActive()) {
            PathNode.cache = p.parent;
            p.node = node;
            p.parent = parent;
            p.next = next;
        }
        else {
            p = new PathNode(node, parent, next);
        }
        return p;
    }
    static release(p, node = NONE) {
        do {
            p.node = NONE;
        } while (p.parent.isActive() && (p = p.parent, node = p.node));
        p.parent = PathNode.cache;
        return node;
    }
    static releaseAndRecount(p, node) {
        do {
            updateCount(p.node);
            p.node = NONE;
        } while (p.parent.isActive() && (p = p.parent, node = p.node));
        p.parent = PathNode.cache;
        return node;
    }
    isActive() {
        return this !== PathNode.NONE;
    }
    isNone() {
        return this === PathNode.NONE;
    }
    replace(node) {
        if (this.parent.isActive()) {
            setChild(this.parent.next, this.parent.node, node);
        }
        this.node = node;
    }
    release() {
        var p = this.parent;
        this.node = NONE;
        this.parent = PathNode.cache;
        PathNode.cache = this;
        return p;
    }
}
PathNode.NONE = new PathNode(NONE, void 0, 0 /* NONE */);
PathNode.NONE.parent = PathNode.NONE;
PathNode.cache = PathNode.NONE;
export function findPath(key, root, compare, group = 0, p) {
    var node = root; // Assumes root has already been assigned. Check for a void root before calling insert().
    if (isUndefined(p)) {
        p = PathNode.NONE;
    }
    do {
        var c = compare(key, node.key);
        if (c < 0) {
            p = PathNode.next(node, p, 1 /* LEFT */);
            node = editLeftChild(group, node);
        }
        else if (c > 0) {
            p = PathNode.next(node, p, 2 /* RIGHT */);
            node = editRightChild(group, node);
        }
        else {
            p = PathNode.next(node, p, 0 /* NONE */);
            node = NONE;
        }
    } while (!isNone(node));
    return p;
}
export function findSuccessor(compare, p, group) {
    p.next = 2 /* RIGHT */;
    var node = editRightChild(group, p.node);
    while (!isNone(node._left)) {
        p = PathNode.next(node, p, 1 /* LEFT */);
        node = editLeftChild(group, node);
    }
    p = PathNode.next(node, p, 0 /* NONE */);
    return p;
}

//# sourceMappingURL=path.js.map
