import { Collection, IndexableCollectionTypeInfo } from '@collectable/core';
import { Node, RedBlackTreeEntry } from './node';
export declare const DEFAULT_COMPARATOR: Comparator<any>;
/**
 * A function that compares two keys and returns a value less than 0 if the first is smaller than the second, a value
 * greater than 0 if the second is smaller than the first, or 0 if they're equal.
 */
export declare type Comparator<K> = (a: K, b: K) => number;
export interface RedBlackTree<K, V> extends Collection<RedBlackTreeEntry<K, V>> {
}
export declare class RedBlackTreeImpl<K, V> implements RedBlackTree<K, V> {
    _owner: number;
    _group: number;
    _compare: Comparator<K>;
    _root: Node<K, V>;
    _size: number;
    readonly '@@type': IndexableCollectionTypeInfo;
    constructor(_owner: number, _group: number, _compare: Comparator<K>, _root: Node<K, V>, _size: number);
    [Symbol.iterator](): IterableIterator<RedBlackTreeEntry<K, V>>;
}
export declare function isRedBlackTree<K, V>(arg: any): arg is RedBlackTreeImpl<K, V>;
export declare function createTree<K, V>(mutable: boolean, comparator?: Comparator<K>): RedBlackTreeImpl<K, V>;
export declare function cloneTree<K, V>(mutable: boolean, tree: RedBlackTreeImpl<K, V>): RedBlackTreeImpl<K, V>;
export declare function cloneAsMutable<K, V>(tree: RedBlackTreeImpl<K, V>): RedBlackTreeImpl<K, V>;
export declare function cloneAsImmutable<K, V>(tree: RedBlackTreeImpl<K, V>): RedBlackTreeImpl<K, V>;
export declare function doneMutating<K, V>(tree: RedBlackTreeImpl<K, V>): RedBlackTreeImpl<K, V>;
